﻿using System;
using Styx.TreeSharp;
using Levelbot.Decorators.Death;
using Bots.Grind;
using Levelbot.Actions.Death;
using Styx;
using Styx.CommonBot.Routines;
using CommonBehaviors.Actions;
using System.Collections.Generic;
using Styx.WoWInternals.WoWObjects;
using Styx.WoWInternals;
using System.Linq;
using System.Threading.Tasks;
using Styx.CommonBot.Coroutines;

namespace AreaHunter.Helpers
{
    public partial class Composites
    {
        // ===========================================================
        // Constants
        // ===========================================================

        // ===========================================================
        // Fields
        // ===========================================================

        private static WoWPoint _StartLocation;
        // ===========================================================
        // Constructors
        // ===========================================================

        // ===========================================================
        // Getter & Setter
        // ===========================================================

        // ===========================================================
        // Methods for/from SuperClass/Interfaces
        // ===========================================================

        // ===========================================================
        // Methods
        // ===========================================================
        public static Composite CreateRoot()
        {
            return new PrioritySelector(
                DeathRoutine(),
                PreCombatRoutine(),
                new ActionRunCoroutine(ctx => LootRoutine()),
                PullRoutine(),
                CombatRoutine(),
                new ActionRunCoroutine(ctx => TargetNewEnemy()),
                new ActionRunCoroutine(ctx => RunHomeRoutine())

            );
        }
        
        // ===========================================================
        // Inner and Anonymous Classes
        // ===========================================================  
        public static async Task<bool> RunHomeRoutine()
        {
            if (GetViableMob(GetLootableMob) == null && GetViableMob(GetAttackableMob) == null)
            {
                await CommonCoroutines.SleepForRandomReactionTime();
                CustomLog.Normal("Everything's dead and nothing to loot. Heading back to the beginning.");
                return MoveTo(_StartLocation);
            }
            return false;
        }

        private static Composite DeathRoutine()
        {
            return new Decorator(ctx => StyxWoW.Me.IsDead || StyxWoW.Me.IsGhost,
                new PrioritySelector(
                    new DecoratorNeedToRelease(new ActionReleaseFromCorpse()),
                    new DecoratorNeedToMoveToCorpse(LevelBot.CreateDeathBehavior()),
                    new DecoratorNeedToTakeCorpse(LevelBot.CreateDeathBehavior()),
                    new ActionSuceedIfDeadOrGhost()
                )
            );
        }

        private static Composite PreCombatRoutine()
        {
            return new Decorator(ctx => !StyxWoW.Me.Combat && !StyxWoW.Me.IsActuallyInCombat,
                new PrioritySelector(
                    new Sequence(
                        RoutineManager.Current.RestBehavior,
                        new ActionAlwaysSucceed()
                    ),
                    new Sequence(
                        RoutineManager.Current.PreCombatBuffBehavior,
                        new ActionAlwaysSucceed()
                    )
                )
            );

        }


        public static Composite PullRoutine()
        {
            return new Decorator(ctx => !StyxWoW.Me.IsFlying,
                new Decorator(ctx => StyxWoW.Me.CurrentTarget != null,
                    new Sequence(
                        RoutineManager.Current.PullBehavior,
                        new ActionAlwaysFail()
                    )
                )
            );
        }

        private static Composite CombatRoutine()
        {
            return new Decorator(ctx => StyxWoW.Me.Combat,
                new PrioritySelector(
                    new Sequence(
                        RoutineManager.Current.HealBehavior,
                        new ActionAlwaysFail()
                    ),
                    new Sequence(
                        RoutineManager.Current.CombatBuffBehavior,
                        new ActionAlwaysFail()
                    ),
                    new Sequence(
                        RoutineManager.Current.CombatBehavior,
                        new ActionAlwaysFail()
                    )
                )
            );
        }

        internal static void OnStart()
        {
            _StartLocation = StyxWoW.Me.Location;
            StyxWoW.Me.ClearTarget();
            
        }

        private static WoWUnit GetViableMob(Func<WoWUnit, bool> FilterMethod)
        {
            IEnumerable<WoWUnit> mobsOfInterestQuery =
                from wowUnit in ObjectManager.GetObjectsOfType<WoWUnit>(false, false)
                where (FilterMethod(wowUnit))
                orderby wowUnit.DistanceSqr
                select wowUnit;

            if (mobsOfInterestQuery.Any())
                return mobsOfInterestQuery.First();
            else return null;
        }
    }
}